@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Code to build assembly instructions and resolve symbolic labels.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    SPIM is distributed under the following conditions:
@@ -20,7 +20,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/inst.c,v 3.47 1999/01/12 05:53:05 larus Exp $
+/* $Header: /Software/SPIM/src/inst.c 10    12/24/00 1:37p Larus $
 */
 
 #include <stdio.h>
@@ -53,6 +53,7 @@ static instruction *mk_r_inst (uint32, int opcode, int rs,
 			       int rt, int rd, int shamt);
 static char* print_imm_expr (char *buf, unsigned int length, imm_expr *expr,
 			     int base_reg);
+static void produce_immediate (imm_expr *expr, int rt, int value_known, int32 value);
 static void sort_name_table (void);
 #else
 static int compare_pair_value ();
@@ -64,6 +65,7 @@ static instruction *mk_i_inst ();
 static instruction *mk_j_inst ();
 static instruction *mk_r_inst ();
 static char* print_imm_expr ();
+static void produce_immediate ();
 static void sort_name_table ();
 #endif
 
@@ -317,30 +319,42 @@ i_type_inst_full_word (opcode, rt, rs, expr, value_known, value)
       int offset;
 
       if (expr->symbol != NULL
-	  && expr->symbol->gp_flag && rs == 0
-	  && (offset = expr->symbol->addr + expr->offset) >= -32*K
-	  && offset < 32*K)
-	i_type_inst_free (opcode, rt, REG_GP, make_imm_expr (offset, NULL, 0));
+	  && expr->symbol->gp_flag
+	  && rs == 0
+	  && IMM_MIN <= (offset = expr->symbol->addr + expr->offset)
+	  && offset <= IMM_MAX)
+	{
+	  i_type_inst_free (opcode, rt, REG_GP, make_imm_expr (offset, NULL, 0));
+	}
       else if (value_known)
 	{
 	  int low, high;
 
 	  high = (value >> 16) & 0xffff;
-	  low = 0xffff & value;
+	  low = value & 0xffff;
 
-	  if (high &&
-	      !(high == 0xffff && (low & 0x8000))) /* load sign-extends */
+	  if (high != 0 &&
+	      !(high == 0xffff && (low & 0x8000)))
 	    {
+	      /* Some of high 16 bits are non-zero */
 	      if (low & 0x8000)
-		high += 1;		/* Adjust since load sign-extends */
+		{
+		  /* Adjust high 16, since load sign-extends low 16*/
+		  high += 1;
+		}
 
 	      i_type_inst_free (Y_LUI_OP, 1, 0, const_imm_expr (high));
 	      if (rs != 0)	/* Base register */
+		{
 		r_type_inst (Y_ADDU_OP, 1, 1, rs);
+		}
 	      i_type_inst_free (opcode, rt, 1, const_imm_expr (low));
 	    }
 	  else
-	    i_type_inst_free (opcode, rt, rs, const_imm_expr (low));
+	    {
+	      /* Special case, sign-extension of low 16 bits sets high to 0xffff */
+	      i_type_inst_free (opcode, rt, rs, const_imm_expr (low));
+	    }
 	}
       else
 	{
@@ -348,7 +362,9 @@ i_type_inst_full_word (opcode, rt, rs, expr, value_known, value)
 	  /* Need to adjust if lower bits are negative */
 	  i_type_inst_free (Y_LUI_OP, 1, 0, upper_bits_of_expr (expr));
 	  if (rs != 0)		/* Base register */
+	    {
 	    r_type_inst (Y_ADDU_OP, 1, 1, rs);
+	    }
 	  i_type_inst_free (opcode, rt, 1, lower_bits_of_expr (expr));
 	}
     }
@@ -364,30 +380,26 @@ i_type_inst_full_word (opcode, rt, rs, expr, value_known, value)
 
       if (expr->symbol != NULL
 	  && expr->symbol->gp_flag && rs == 0
-	  && (offset = expr->symbol->addr + expr->offset) >= -32*K
-	  && offset < 32*K)
+	  && IMM_MIN <= (offset = expr->symbol->addr + expr->offset)
+	  && offset <= IMM_MAX)
+	{
 	i_type_inst_free ((opcode == Y_LUI_OP ? Y_ADDIU_OP : opcode),
-		       rt, REG_GP, make_imm_expr (offset, NULL, 0));
+			  rt, REG_GP, make_imm_expr (offset, NULL, 0));
+	}
       else
 	{
 	  /* Use $at */
 	  if ((opcode == Y_ORI_OP
-	       || opcode == Y_ADDI_OP || opcode == Y_ADDIU_OP
+	       || opcode == Y_ADDI_OP
+	       || opcode == Y_ADDIU_OP
 	       || opcode == Y_LUI_OP)
 	      && rs == 0)
 	    {
-	      if (value_known && (value & 0xffff) == 0)
-		i_type_inst_free (Y_LUI_OP, rt, 0, upper_bits_of_expr (expr));
-	      else
-		{
-		  i_type_inst_free (Y_LUI_OP, 1, 0, upper_bits_of_expr (expr));
-		  i_type_inst_free (Y_ORI_OP, rt, 1, lower_bits_of_expr(expr));
-		}
+	      produce_immediate(expr, rt, value_known, value);
 	    }
 	  else
 	    {
-	      i_type_inst_free (Y_LUI_OP, 1, 0, upper_bits_of_expr (expr));
-	      i_type_inst_free (Y_ORI_OP, 1, 1, lower_bits_of_expr (expr));
+	      produce_immediate(expr, 1, value_known, value);
 	      r_type_inst (imm_op_to_op (opcode), rt, rs, 1);
 	    }
 	}
@@ -395,6 +407,34 @@ i_type_inst_full_word (opcode, rt, rs, expr, value_known, value)
 }
 
 
+#ifdef __STDC__
+static void
+produce_immediate (imm_expr *expr, int rt, int value_known, int32 value)
+#else
+static void
+produce_immediate (expr, rt, value_known, value)
+     imm_expr *expr;
+     int rt;
+     int value_known;
+     int32 value;
+#endif
+{
+  if (value_known && (value & 0xffff) == 0)
+    {
+      i_type_inst_free (Y_LUI_OP, rt, 0, upper_bits_of_expr (expr));
+    }
+  else if (value_known && (value & 0xffff0000) == 0)
+    {
+      i_type_inst_free (Y_ORI_OP, rt, 0, lower_bits_of_expr (expr));
+    }
+  else
+    {
+      i_type_inst_free (Y_LUI_OP, 1, 0, upper_bits_of_expr (expr));
+      i_type_inst_free (Y_ORI_OP, rt, 1, lower_bits_of_expr(expr));
+    }
+}
+
+
 /* Return a jump-type instruction with the given OPCODE and TARGET
    fields. NB, even the immediate value may not fit in the field, this
    routine will not produce more than one instruction. */
@@ -1250,9 +1290,9 @@ eval_imm_expr (expr)
       value = 0;
     }
   if (expr->bits > 0)
-    return ((value >> 16) & 0xffff); /* Use upper bits of result */
+    return ((value >> 16) & 0xffff);  /* Use upper bits of result */
   else if (expr->bits < 0)
-    return (value & 0xffff);	/* Use lower bits */
+    return (value & 0xffff);	      /* Use lower bits */
   else
     return (value);
 }
