@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Description of a SPIM S20 instruction.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    SPIM is distributed under the following conditions:
@@ -20,7 +20,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/inst.h,v 3.15 1999/01/08 00:32:54 larus Exp $
+/* $Header: /Software/SPIM/src/inst.h 7     12/24/00 1:37p Larus $
 */
 
 
@@ -134,6 +134,13 @@ typedef struct inst_s
 #define COND_LT		0x4
 #define COND_IN		0x8
 
+/* Minimum and maximum values that fit in instruction's imm field */
+#define IMM_MIN -(1<<15)
+#define IMM_MAX ((1<<15)-1)
+
+#define UIMM_MIN  0
+#define UIMM_MAX  ((1<<16)-1)
+
 
 
 /* Raise an exception! */
