@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Code to create, maintain and access memory.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    SPIM is distributed under the following conditions:
@@ -20,7 +20,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/mem.c,v 3.41 1999/01/08 00:32:54 larus Exp $
+/* $Header: /Software/SPIM/src/mem.c 8     12/24/00 1:37p Larus $
 */
 
 
@@ -383,7 +383,13 @@ bad_mem_read (addr, mask, dest)
 	return (0xffff & tmp);
 
       case 0x3:
-	return (ENCODING (text_seg [(addr - TEXT_BOT) >> 2]));
+	{
+	instruction *inst = text_seg [(addr - TEXT_BOT) >> 2];
+	if (inst == NULL)
+	  return 0;
+	else
+	  return (ENCODING (inst));
+	}
 
       default:
 	run_error ("Bad mask (0x%x) in bad_mem_read\n", mask);
