@@ -2,7 +2,7 @@
    Execute SPIM syscalls, both in simulator and bare mode.
    Execute MIPS syscalls in bare mode, when running on MIPS systems.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    Improved by Emin Gun Sirer.
@@ -24,7 +24,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/mips-syscall.c,v 1.35 1999/01/08 00:32:54 larus Exp $ */
+/* $Header: /Software/SPIM/src/mips-syscall.c 7     12/24/00 1:37p Larus $ */
 
 
 #include <stdio.h>
@@ -417,12 +417,12 @@ do_syscall ()
 	  {
 	    float val = FPR_S (REG_FA0);
 
-	    write_output (console_out, "%f", val);
+	    write_output (console_out, "%.18f", val);
 	    break;
 	  }
 
 	case PRINT_DOUBLE_SYSCALL:
-	  write_output (console_out, "%f", FPR[REG_FA0/2]);
+	  write_output (console_out, "%.18g", FPR[REG_FA0/2]);
 	  break;
 
 	case PRINT_STRING_SYSCALL:
@@ -674,9 +674,9 @@ do_syscall ()
 	    case SYS_brk:
 	      /* Round up to 4096 byte (page) boundary */
 	      if ( ( (int) R[REG_A0] - (int) data_top) > 0)
-		expand_data (ROUND (R[REG_A0], 4096)- (int)data_top);
+		expand_data (ROUND_UP (R[REG_A0], 4096)- (int)data_top);
 	      R[REG_RES] = program_break;
-	      program_break = ROUND (R[REG_A0], 4096);
+	      program_break = ROUND_UP (R[REG_A0], 4096);
 	      R[REG_ERR] = 0;
 	      break;
 
