@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Execute SPIM instructions.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
    Changes for DOS and Windows versions by David A. Carley (dac@cs.wisc.edu)
 
@@ -21,7 +21,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/run.c,v 3.48 1999/01/08 00:32:54 larus Exp $
+/* $Header: /Software/SPIM/src/run.c 8     12/24/00 1:37p Larus $
 */
 
 
@@ -85,7 +85,7 @@ static void long_multiply ();
 #define BRANCH_INST(TEST, TARGET) {if (TEST)				\
 				    {					\
 				      mem_addr target = (TARGET);	\
-				      if (bare_machine)			\
+				      if (delayed_branches)		\
 					/* +4 since jump in delay slot */\
 					target += BYTES_PER_WORD;	\
 				      JUMP_INST (target)		\
@@ -93,7 +93,7 @@ static void long_multiply ();
 				  }
 
 
-#define JUMP_INST(TARGET) {if (bare_machine)				\
+#define JUMP_INST(TARGET) {if (delayed_branches)			\
 			    run_spim (PC + BYTES_PER_WORD, 1, display);	\
 			  /* -4 since PC is bumped after this inst */	\
 			  PC = (TARGET) - BYTES_PER_WORD;		\
@@ -116,7 +116,7 @@ static void long_multiply ();
 			      }
 
 
-#define DO_DELAYED_UPDATE() if (bare_machine)				\
+#define DO_DELAYED_UPDATE() if (delayed_loads)				\
 			       {					\
 				 /* Check for delayed updates */	\
 				 if (delayed_addr2 != NULL)		\
@@ -184,7 +184,7 @@ run_spim (initial_PC, steps_to_run, display)
 	  if (exception_occurred)
 	    {
 	      exception_occurred = 0;
-	      EPC = PC & 0xfffffffc; /* Round down */
+	      EPC = ROUND_DOWN (PC, BYTES_PER_WORD);
 	      handle_exception();
 	      continue;
 	    }
@@ -274,7 +274,7 @@ run_spim (initial_PC, steps_to_run, display)
 	      break;
 
 	    case Y_BGEZAL_OP:
-	      if (bare_machine)
+	      if (delayed_branches)
 		R[31] = PC + 2 * BYTES_PER_WORD;
 	      else
 		R[31] = PC + BYTES_PER_WORD;
@@ -298,7 +298,7 @@ run_spim (initial_PC, steps_to_run, display)
 	      break;
 
 	    case Y_BLTZAL_OP:
-	      if (bare_machine)
+	      if (delayed_branches)
 		R[31] = PC + 2 * BYTES_PER_WORD;
 	      else
 		R[31] = PC + BYTES_PER_WORD;
@@ -358,7 +358,7 @@ run_spim (initial_PC, steps_to_run, display)
 	      break;
 
 	    case Y_JAL_OP:
-	      if (bare_machine)
+	      if (delayed_branches)
 		R[31] = PC + 2 * BYTES_PER_WORD;
 	      else
 		R[31] = PC + BYTES_PER_WORD;
@@ -369,7 +369,7 @@ run_spim (initial_PC, steps_to_run, display)
 	      {
 		mem_addr tmp = R[RS (inst)];
 
-		if (bare_machine)
+		if (delayed_branches)
 		  R[RD (inst)] = PC + 2 * BYTES_PER_WORD;
 		else
 		  R[RD (inst)] = PC + BYTES_PER_WORD;
@@ -442,30 +442,30 @@ run_spim (initial_PC, steps_to_run, display)
 		      break;
 
 		    case 1:
-		      R[RT (inst)] = (word & 0xffffff) << 8 | (reg_val & 0xff);
+		      R[RT (inst)] = ((word & 0xffffff) << 8) | (reg_val & 0xff);
 		      break;
 
 		    case 2:
-		      R[RT (inst)] = (word & 0xffff) << 16 | (reg_val & 0xffff);
+		      R[RT (inst)] = ((word & 0xffff) << 16) | (reg_val & 0xffff);
 		      break;
 
 		    case 3:
-		      R[RT (inst)] = (word & 0xff) << 24 | (reg_val & 0xffffff);
+		      R[RT (inst)] = ((word & 0xff) << 24) | (reg_val & 0xffffff);
 		      break;
 		    }
 #else
 		switch (byte)
 		  {
 		  case 0:
-		    R[RT (inst)] = (word & 0xff) << 24 | (reg_val & 0xffffff);
+		    R[RT (inst)] = ((word & 0xff) << 24) | (reg_val & 0xffffff);
 		    break;
 
 		  case 1:
-		    R[RT (inst)] = (word & 0xffff) << 16 | (reg_val & 0xffff);
+		    R[RT (inst)] = ((word & 0xffff) << 16) | (reg_val & 0xffff);
 		    break;
 
 		  case 2:
-		    R[RT (inst)] = (word & 0xffffff) << 8 | (reg_val & 0xff);
+		    R[RT (inst)] = ((word & 0xffffff) << 8) | (reg_val & 0xff);
 		    break;
 
 		  case 3:
