@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Misc. routines for SPIM.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
    Changes for DOS and Windows versions by David A. Carley (dac@cs.wisc.edu)
 
@@ -21,7 +21,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/spim-utils.c,v 3.58 1999/01/08 00:32:54 larus Exp $
+/* $Header: /Software/SPIM/src/spim-utils.c 8     12/24/00 1:37p Larus $
 */
 
 
@@ -148,7 +148,7 @@ write_startup_message ()
 {
   write_output (message_out, "SPIM %s\n", SPIM_VERSION);
   write_output (message_out,
-		"Copyright 1990-1998 by James R. Larus (larus@cs.wisc.edu).\n");
+		"Copyright 1990-2000 by James R. Larus (larus@cs.wisc.edu).\n");
   write_output (message_out, "All Rights Reserved.\n");
 #if defined(DJGPP) || defined(WIN32)
   write_output (message_out, "DOS and Windows ports by David A. Carley (dac@cs.wisc.edu).\n");
@@ -195,7 +195,7 @@ read_assembly_file (name)
      char *name;
 #endif
 {
-  FILE *file = fopen (name, "r");;
+  FILE *file = fopen (name, "rt");
 
   source_file = 1;
   if (file == NULL)
