@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Terminal interface for SPIM simulator.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
    Changes for DOS and Windows versions by David A. Carley (dac@cs.wisc.edu)
 
@@ -21,7 +21,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/spim.c,v 3.65 1999/01/08 00:32:54 larus Exp $
+/* $Header: /Software/SPIM/src/spim.c 10    1/14/01 12:41p Larus $
 */
 
 
@@ -90,6 +90,7 @@ static void flush_to_newline (void);
 static int get_opt_int (void);
 static int parse_spim_command (FILE *file, int redo);
 static int print_reg (int reg_no, int type_code);
+static void print_all_regs (int hex_flag);
 static int read_assembly_command (void);
 static int str_prefix (char *s1, char *s2, int min_match);
 static void top_level (void);
@@ -101,6 +102,7 @@ static void flush_to_newline ();
 static int get_opt_int ();
 static int parse_spim_command ();
 static int print_reg ();
+static void print_all_regs ();
 static int read_assembly_command ();
 static int str_prefix ();
 static void top_level ();
@@ -114,6 +116,8 @@ static int read_token ();
 jmp_buf spim_top_level_env;	/* For ^C */
 
 int bare_machine;		/* Non-Zero => simulate bare machine */
+int delayed_branches;		/* Non-Zero => simulate delayed branches */
+int delayed_loads;		/* Non-Zero => simulate delayed loads */
 int accept_pseudo_insts;	/* Non-Zero => parse pseudo instructions  */
 int quiet;			/* Non-Zero => no warning messages */
 int source_file;		/* Non-Zero => program is source, not binary */
@@ -154,6 +158,8 @@ main (argc, argv)
   message_out.f = stdout;
 
   bare_machine = 0;
+  delayed_branches = 0;
+  delayed_loads = 0;
   accept_pseudo_insts = 1;
   quiet = 0;
   source_file = 0;
@@ -167,16 +173,33 @@ main (argc, argv)
 
   for (i = 1; i < argc; i++)
     {
-#ifdef DJGPP
-      /* On DOS, support "/option" as well as "-option" */
+#ifdef WIN32
+      /* On Windows, support "/option" as well as "-option" */
       if (argv [i][0] == '/')
 	argv [i][0] = '-';
 #endif
 
       if (streq (argv [i], "-bare"))
-	bare_machine = 1, quiet = 1;
+	{
+	  bare_machine = 1;
+	  delayed_branches = 1;
+	  delayed_loads = 1;
+	  quiet = 1;
+	}
       else if (streq (argv [i], "-asm"))
-	bare_machine = 0;
+	{
+	  bare_machine = 0;
+	  delayed_branches = 0;
+	  delayed_loads = 0;
+	}
+      else if (streq (argv [i], "-delayed_branches"))
+	{
+	  delayed_branches = 1;
+	}
+      else if (streq (argv [i], "-delayed_loads"))
+	{
+	  delayed_loads = 1;
+	}
       else if (streq (argv [i], "-pseudo"))
 	accept_pseudo_insts = 1;
       else if (streq (argv [i], "-nopseudo"))
@@ -239,18 +262,18 @@ main (argc, argv)
       console_to_program ();
       initialize_run_stack (argc - argv_ptr, &argv[argv_ptr]);
       if (!setjmp (spim_top_level_env))
-      {
-	char *undefs = undefined_symbol_string ();
-	if (undefs != NULL)
-	  {
-	    write_output (message_out, "The following symbols are undefined:\n");
-	    write_output (message_out, undefs);
-	    write_output (message_out, "\n");
-	    free (undefs);
-	  }
-	run_program (find_symbol_address (DEFAULT_RUN_LOCATION),
-		     DEFAULT_RUN_STEPS, 0, 0);
-      }
+	{
+	  char *undefs = undefined_symbol_string ();
+	  if (undefs != NULL)
+	    {
+	      write_output (message_out, "The following symbols are undefined:\n");
+	      write_output (message_out, undefs);
+	      write_output (message_out, "\n");
+	      free (undefs);
+	    }
+	  run_program (find_symbol_address (DEFAULT_RUN_LOCATION),
+		       DEFAULT_RUN_STEPS, 0, 0);
+	}
       console_to_spim ();
     }
 
@@ -302,23 +325,25 @@ int arg;
 
 /* SPIM commands */
 
-#define UNKNOWN_CMD		0
-#define EXIT_CMD		1
-#define READ_CMD		2
-#define RUN_CMD			3
-#define STEP_CMD		4
-#define PRINT_CMD		5
-#define PRINT_SYM_CMD		6
-#define REINITIALIZE_CMD	7
-#define ASM_CMD			8
-#define REDO_CMD		9
-#define NOP_CMD			10
-#define HELP_CMD		11
-#define CONTINUE_CMD		12
-#define SET_BKPT_CMD		13
-#define DELETE_BKPT_CMD		14
-#define LIST_BKPT_CMD		15
-
+enum {
+  UNKNOWN_CMD = 0,
+  EXIT_CMD,
+  READ_CMD,
+  RUN_CMD,
+  STEP_CMD,
+  PRINT_CMD,
+  PRINT_SYM_CMD,
+  PRINT_ALL_REGS_CMD,
+  REINITIALIZE_CMD,
+  ASM_CMD,
+  REDO_CMD,
+  NOP_CMD,
+  HELP_CMD,
+  CONTINUE_CMD,
+  SET_BKPT_CMD,
+  DELETE_BKPT_CMD,
+  LIST_BKPT_CMD
+};
 
 /* Parse a SPIM command from the FILE and execute it.  If REDO is non-zero,
    don't read a new command; just rexecute the previous one.
@@ -480,6 +505,18 @@ parse_spim_command (file, redo)
       prev_cmd = NOP_CMD;
       return (0);
 
+    case PRINT_ALL_REGS_CMD:
+      {
+	int hex_flag = 0;
+	int token = (redo ? prev_token : read_token ());
+	if (token == Y_ID && streq(yylval.p, "hex"))
+	  hex_flag = 1;
+	print_all_regs (hex_flag);
+	if (!redo) flush_to_newline ();
+	prev_cmd = NOP_CMD;
+	return (0);
+      }
+
     case REINITIALIZE_CMD:
       flush_to_newline ();
       initialize_world (load_trap_handler ? trap_file : NULL);
@@ -521,7 +558,11 @@ parse_spim_command (file, redo)
       write_output (message_out,
 		    "print ADDR -- Print contents of memory at ADDRESS\n");
       write_output (message_out,
-		    "print_symbols -- Print a list of all symbols\n");
+		    "print_symbols -- Print all global symbols\n");
+      write_output (message_out,
+		    "print_all_regs -- Print all MIPS registers\n");
+      write_output (message_out,
+		    "print_all_regs hex -- Print all MIPS registers in hex\n");
       write_output (message_out,
 		    "reinitialize -- Clear the memory and registers\n");
       write_output (message_out,
@@ -599,8 +640,10 @@ read_assembly_command ()
     return (EXIT_CMD);
   else if (str_prefix ((char *) yylval.p, "print", 1))
     return (PRINT_CMD);
-  else if (str_prefix ((char *) yylval.p, "print_symbols", 6))
+  else if (str_prefix ((char *) yylval.p, "print_symbols", 7))
     return (PRINT_SYM_CMD);
+  else if (str_prefix ((char *) yylval.p, "print_all_regs", 7))
+    return (PRINT_ALL_REGS_CMD);
   else if (str_prefix ((char *) yylval.p, "run", 2))
     return (RUN_CMD);
   else if (str_prefix ((char *) yylval.p, "read", 2))
@@ -767,6 +810,24 @@ print_reg (reg_no, type_code)
   return (1);
 }
 
+#define MAX_BUF_LEN 32000
+
+#ifdef __STDC__
+static void
+print_all_regs (int hex_flag)
+#else
+static void
+print_all_regs (hex_flag)
+     int hex_flag;
+#endif
+{
+  int max_buf_len = MAX_BUF_LEN;
+  char buf[MAX_BUF_LEN];
+  int count;
+  registers_as_string (buf, &max_buf_len, &count, hex_flag, hex_flag);
+  write_output (message_out, "%s\n", buf);
+}
+
 
 
 /* Print an error message. */
@@ -974,7 +1035,7 @@ console_to_program ()
 #endif
 
       params.c_iflag |= IGNBRK|IGNPAR;
-      params.c_oflag &= ~OPOST;
+      /*params.c_oflag &= ~OPOST;*/
       params.c_cflag &= ~PARENB;
       params.c_cflag |= CREAD|CS8;
       params.c_lflag = 0;
@@ -992,8 +1053,8 @@ console_to_program ()
       saved_console_state.sg_flags = (flags | RAW) & ~(CRMOD|ECHO);
       ioctl ((int) console_in.i, TIOCSETP, (char *) &saved_console_state);
       saved_console_state.sg_flags = flags;
-      console_state_saved = 1;
 #endif
+      console_state_saved = 1;
     }
 }
 
@@ -1011,9 +1072,9 @@ console_to_spim ()
   if (mapped_io && console_state_saved)
 #ifdef USE_TERMIO
 #ifdef DJGPP
-    tcgetattr((int)console_in.i, &saved_console_state);
+    tcsetattr((int)console_in.i, &saved_console_state);
 #else
-    ioctl ((int) console_in.i, TCGETA, (char *) &saved_console_state);
+    ioctl ((int) console_in.i, TCSETA, (char *) &saved_console_state);
 #endif
 #else
     ioctl ((int) console_in.i, TIOCSETP, (char *) &saved_console_state);
@@ -1030,6 +1091,7 @@ int
 console_input_available ()
 #endif
 {
+#ifndef __CYGWIN32__
   fd_set fdset;
   struct timeval timeout;
 
@@ -1042,6 +1104,7 @@ console_input_available ()
       return (select (sizeof (fdset) * 8, &fdset, NULL, NULL, &timeout));
     }
   else
+#endif
     return (0);
 }
 
