@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Definitions for the SPIM S20.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
    Changes for DOS and Windows versions by David A. Carley (dac@cs.wisc.edu)
 
@@ -21,9 +21,10 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/spim.h,v 3.30 1999/01/08 00:32:54 larus Exp $
+/* $Header: /Software/SPIM/src/spim.h 9     12/24/00 1:37p Larus $
 */
 
+
 #ifndef NULL
 #define NULL 0
 #endif
@@ -42,8 +43,8 @@ typedef union {int i; void* p;} intptr_union;
 
 /* Round V to next greatest B boundary */
 
-#define ROUND(V, B) (((int) V + (B-1)) & ~(B-1))
-
+#define ROUND_UP(V, B) (((int) V + (B-1)) & ~(B-1))
+#define ROUND_DOWN(V, B) (((int) V) & ~(B-1))
 
 /* Sign-extend an int16 to an int32 */
 
@@ -251,6 +252,8 @@ void write_output ();
 
 extern int bare_machine;	/* Non-zero => simulate bare machine */
 extern int accept_pseudo_insts;	/* Non-Zero => parse pseudo instructions  */
+extern int delayed_branches;	/* Non-zero => simulate delayed branches */
+extern int delayed_loads;	/* Non-zero => simulate delayed loads */
 extern int quiet;		/* Non-zero => no warning messages */
 extern int source_file;		/* Non-zero => program is source, not binary */
 extern char *trap_file;		/* Path of file containing trap handler */
