@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Code to maintain symbol table to resolve symbolic labels.
 
-   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    SPIM is distributed under the following conditions:
@@ -20,7 +20,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/sym-tbl.c,v 3.21 1999/01/08 00:32:54 larus Exp $
+/* $Header: /Software/SPIM/src/sym-tbl.c 12    12/24/00 1:37p Larus $
 */
 
 
@@ -387,12 +387,11 @@ resolve_a_label_sub (sym, inst, pc)
 	    {
 	      int val;
 
-	      /* Drop low two bits since instructions are on word
-		 boundaries. */
-	      val = eval_imm_expr (EXPR (inst));
-	      val >>= 2;	/* Seperate to force sign-extension */
+	      /* Drop low two bits since instructions are on word boundaries. */
+	      val = SIGN_EX (eval_imm_expr (EXPR (inst)));   /* 16->32 bits */
+	      val = (val >> 2) & 0xffff;	    /* right shift, 32->16 bits */
 
-	      if (bare_machine)	/* Delayed branch */
+	      if (delayed_branches)
 		val -= 1;
 
 	      value = val;
@@ -400,8 +399,7 @@ resolve_a_label_sub (sym, inst, pc)
 	    }
 	  else if (opcode_is_jump (OPCODE (inst)))
 	    {
-	      /* Drop low two bits since instructions are on word
-		 boundaries. */
+	      /* Drop low two bits since instructions are on word boundaries. */
 	      value = eval_imm_expr (EXPR (inst)) >> 2;
 	      mask = 0x03fffffff;
 	    }
@@ -411,27 +409,20 @@ resolve_a_label_sub (sym, inst, pc)
 	      value = eval_imm_expr (EXPR (inst));
 	      mask = 0xffff;
 
-	      /* LW/SW sign extends offset.  Add 1 to high 16 bits
-	         to compensate. */
 	      if (value & 0x8000)
-	      {
-		instruction* prev_inst;
-		READ_MEM_INST (prev_inst, pc - BYTES_PER_WORD);
-		if (OPCODE (prev_inst) == Y_LUI_OP
-		    && EXPR (inst)->symbol == EXPR (prev_inst)->symbol)
 		{
-		  if (IMM (prev_inst) == 0)
-		  {
-		    /* Have not yet resolved label */
-		    EXPR (prev_inst)->offset += 0x10000;
-		  }
-		  else
-		  {
-		    /* Already resolved label */
-		    IMM (prev_inst) += 1;
-		  }
+  		  /* LW/SW sign extends offset. Compensate by adding 1 to high 16 bits. */
+		  instruction* prev_inst;
+		  READ_MEM_INST (prev_inst, pc - BYTES_PER_WORD);
+		  if (OPCODE (prev_inst) == Y_LUI_OP
+		      && EXPR (inst)->symbol == EXPR (prev_inst)->symbol
+		      && IMM (prev_inst) == 0)
+		    {
+		      /* Check that previous instruction was LUI and it has no immediate,
+			 otherwise it will have compensated for sign-extension */
+		      EXPR (prev_inst)->offset += 0x10000;
+		    }
 		}
-	      }
 	    }
 	  else
 	    {
